@@ -3,7 +3,7 @@
  * :folding=explicit:collapseFolds=1:
  *
  * Copyright (C) 2003, 2005 Slava Pestov
- * Portions copyright (C) 2006 Matthieu Casanova
+ * Portions copyright (C) 2006, 2011 Matthieu Casanova
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,6 +34,8 @@ import org.gjt.sp.util.Log;
 import org.gjt.sp.util.XMLUtilities;
 import org.gjt.sp.util.IOUtilities;
 
+import javax.swing.*;
+
 /**
  * The basic KillRing of jEdit.
  * @author Matthieu Casanova
@@ -68,6 +70,16 @@ class JEditKillRing extends KillRing
 		{
 			killringXML.load(handler);
 		}
+		catch (OutOfMemoryError oem)
+		{
+			Log.log(Log.ERROR, this, "Unable to load entire Killring, too low memory, increase your jvm max heap size");
+			int confirm = GUIUtilities.confirm(null, "killring.load-memoryerror", null, JOptionPane.YES_NO_OPTION,
+					JOptionPane.ERROR_MESSAGE);
+			if (confirm == JOptionPane.NO_OPTION)
+			{
+				System.exit(-1);
+			}
+		}
 		catch (IOException ioe)
 		{
 			Log.log(Log.ERROR, this, ioe);
