@@ -37,12 +37,13 @@ import org.gjt.sp.jedit.gui.DockableWindowManager;
 import org.gjt.sp.jedit.io.*;
 import org.gjt.sp.jedit.*;
 import org.gjt.sp.util.Log;
+import org.gjt.sp.util.ThreadUtilities;
 //}}}
 
 /**
  * VFS browser tree view.
  * @author Slava Pestov
- * @version $Id: BrowserView.java 16560 2009-11-29 19:06:17Z kerik-sf $
+ * @version $Id: BrowserView.java 18572 2010-09-17 10:57:58Z kpouer $
  */
 class BrowserView extends JPanel
 {
@@ -81,7 +82,7 @@ class BrowserView extends JPanel
 			parentScroller, tableScroller);
 		splitPane.setOneTouchExpandable(true);
 
-		SwingUtilities.invokeLater(new Runnable()
+		EventQueue.invokeLater(new Runnable()
 		{
 			public void run()
 			{
@@ -155,6 +156,14 @@ class BrowserView extends JPanel
 	//{{{ loadDirectory() method
 	public void loadDirectory(Object node, String path,
 		boolean addToHistory)
+	{
+		loadDirectory(node, path, addToHistory, null);
+	} //}}}
+
+
+	//{{{ loadDirectory() method
+	public void loadDirectory(final Object node, String path,
+		final boolean addToHistory, final Runnable delayedAWTTask)
 	{
 		path = MiscUtilities.constructPath(browser.getDirectory(),path);
 		VFS vfs = VFSManager.getVFSForPath(path);
@@ -169,12 +178,18 @@ class BrowserView extends JPanel
 				new LoadingPlaceholder() });
 		}
 
-		Object[] loadInfo = new Object[2];
-
-		VFSManager.runInWorkThread(new BrowserIORequest(
-			BrowserIORequest.LIST_DIRECTORY,browser,
-			session,vfs,path,null,loadInfo));
-		browser.directoryLoaded(node,loadInfo,addToHistory);
+		final Object[] loadInfo = new Object[2];
+		Runnable awtRunnable = new Runnable()
+		{
+			public void run()
+			{
+				browser.directoryLoaded(node,loadInfo,addToHistory);
+				if (delayedAWTTask != null)
+					delayedAWTTask.run();
+			}
+		};
+		ThreadUtilities.runInBackground(new ListDirectoryBrowserTask(browser,
+			session, vfs, path, null, loadInfo, awtRunnable));
 	} //}}}
 
 	//{{{ directoryLoaded() method
@@ -299,8 +314,7 @@ class BrowserView extends JPanel
 
 		if(browserDir.startsWith(FileRootsVFS.PROTOCOL)
 			&& MiscUtilities.isURL(path)
-			&& !MiscUtilities.getProtocolOfURL(path)
-			.equals("file"))
+			&& !"file".equals(MiscUtilities.getProtocolOfURL(path)))
 			return;
 
 		table.maybeReloadDirectory(path);
@@ -369,7 +383,7 @@ class BrowserView extends JPanel
 				// we use SwingUtilities.invokeLater()
 				// so that the action is executed before
 				// the popup is hidden.
-				SwingUtilities.invokeLater(new Runnable()
+				EventQueue.invokeLater(new Runnable()
 				{
 					public void run()
 					{
@@ -637,7 +651,7 @@ class BrowserView extends JPanel
 						parentDirectories.setSelectedIndex(++row);
 					break;
 				case KeyEvent.VK_LEFT:
-					if ((evt.getModifiers() & KeyEvent.ALT_MASK)>0)
+					if ((evt.getModifiers() & InputEvent.ALT_MASK)>0)
 					{
 						evt.consume();
 						browser.previousDirectory();
@@ -645,7 +659,7 @@ class BrowserView extends JPanel
 					else super.processEvent(evt);
 					break;
 				case KeyEvent.VK_RIGHT:
-					if ((evt.getModifiers() & KeyEvent.ALT_MASK)>0)
+					if ((evt.getModifiers() & InputEvent.ALT_MASK)>0)
 					{
 						evt.consume();
 						browser.nextDirectory();
@@ -654,7 +668,7 @@ class BrowserView extends JPanel
 					break;
 				case KeyEvent.VK_TAB:
 					evt.consume();
-					if ((evt.getModifiers() & KeyEvent.SHIFT_MASK) > 0)
+					if ((evt.getModifiers() & InputEvent.SHIFT_MASK) > 0)
 						browser.focusOnDefaultComponent();
 					else
 						table.requestFocus();
