@@ -4,6 +4,7 @@
  * :folding=explicit:collapseFolds=1:
  *
  * Copyright (C) 2006 Björn Kautler
+ * Portions copyright (C) 2010 Matthieu Casanova
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,24 +24,20 @@
 package org.gjt.sp.jedit.options;
 
 //{{{ Imports
-import java.awt.Dimension;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Vector;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
 import javax.swing.Box;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
 import javax.swing.JComboBox;
-import javax.swing.JScrollPane;
 import javax.swing.JTextField;
-import javax.swing.border.TitledBorder;
-import javax.swing.event.TableModelEvent;
-import javax.swing.event.TableModelListener;
 import org.gjt.sp.jedit.AbstractOptionPane;
+import org.gjt.sp.jedit.gui.PingPongList;
 import org.gjt.sp.jedit.jEdit;
 import org.gjt.sp.jedit.buffer.JEditBuffer;
-import org.gjt.sp.jedit.gui.JCheckBoxList;
-import org.gjt.sp.jedit.gui.JCheckBoxList.Entry;
 import org.gjt.sp.util.StandardUtilities;
 import static java.awt.GridBagConstraints.BOTH;
 import static java.util.Arrays.sort;
@@ -58,8 +55,9 @@ import static org.gjt.sp.jedit.MiscUtilities.getEncodings;
  * Encodings options.
  * 
  * @author Björn Kautler
+ * @author Matthieu Casanova
  * @since jEdit 4.3pre6
- * @version $Id: EncodingsOptionPane.java 16347 2009-10-14 10:37:09Z kpouer $
+ * @version $Id: EncodingsOptionPane.java 17766 2010-05-08 12:39:07Z k_satoda $
  */
 public class EncodingsOptionPane extends AbstractOptionPane
 {
@@ -68,10 +66,10 @@ public class EncodingsOptionPane extends AbstractOptionPane
 	private JCheckBox encodingAutodetect;
 	private JTextField encodingDetectors;
 	private JTextField fallbackEncodings;
-	
-	private JCheckBoxList encodingsList;
+
 	private JButton selectAllButton;
 	private JButton selectNoneButton;
+	private PingPongList<String> pingPongList;
 	//}}}
 
 	//{{{ EncodingsOptionPane constructor
@@ -102,7 +100,7 @@ public class EncodingsOptionPane extends AbstractOptionPane
 		
 		// Encoding detectors
 		encodingDetectors = new JTextField(jEdit.getProperty(
-			"encodingDetectors","BOM XML-PI"));
+			"encodingDetectors",""));
 		addComponent(jEdit.getProperty("options.general.encodingDetectors"),encodingDetectors);
 
 		// Fallback Encodings
@@ -115,25 +113,22 @@ public class EncodingsOptionPane extends AbstractOptionPane
 		// Encodings to display
 		encodings = getEncodings(false);
 		sort(encodings,new StandardUtilities.StringCompare<String>(true));
-		Vector<Entry> encodingEntriesVector = new Vector<Entry>();
-		boolean enableSelectAll = false;
-		boolean enableSelectNone = false;
+		List<String> availableEncodings = new ArrayList<String>();
+		List<String> selectedEncodings = new ArrayList<String>();
 		for (String encoding : encodings)
 		{
 			boolean selected = !getBooleanProperty("encoding.opt-out."+encoding,false);
-			enableSelectAll = enableSelectAll || !selected;
-			enableSelectNone = enableSelectNone || selected;
-			encodingEntriesVector.add(new Entry(selected,encoding));
+			if (selected)
+				selectedEncodings.add(encoding);
+			else
+				availableEncodings.add(encoding);
 		}
-		encodingsList = new JCheckBoxList(encodingEntriesVector);
-		encodingsList.getModel().addTableModelListener(new TableModelHandler());
-		JScrollPane encodingsScrollPane = new JScrollPane(encodingsList);
-		encodingsScrollPane.setBorder(
-			new TitledBorder(getProperty("options.encodings.selectEncodings")));
-		Dimension d = encodingsList.getPreferredSize();
-		d.height = Math.min(d.height,200);
-		encodingsScrollPane.setPreferredSize(d);
-		addComponent(encodingsScrollPane,BOTH);
+		pingPongList = new PingPongList<String>(availableEncodings, selectedEncodings);
+		pingPongList.setLeftTitle(getProperty("options.encodings.available"));
+		pingPongList.setRightTitle(getProperty("options.encodings.selected"));
+		pingPongList.setLeftTooltip(getProperty("options.encodings.available.tooltip"));
+		pingPongList.setRightTooltip(getProperty("options.encodings.selected.tooltip"));
+		addComponent(pingPongList,BOTH);
 
 		// Select All/None Buttons
 		Box buttonsBox = createHorizontalBox();
@@ -142,13 +137,13 @@ public class EncodingsOptionPane extends AbstractOptionPane
 		ActionHandler actionHandler = new ActionHandler();
 		selectAllButton = new JButton(getProperty("options.encodings.selectAll"));
 		selectAllButton.addActionListener(actionHandler);
-		selectAllButton.setEnabled(enableSelectAll);
+		selectAllButton.setEnabled(pingPongList.getLeftSize() != 0);
 		buttonsBox.add(selectAllButton);
 		buttonsBox.add(createHorizontalStrut(12));
 
 		selectNoneButton = new JButton(getProperty("options.encodings.selectNone"));
 		selectNoneButton.addActionListener(actionHandler);
-		selectNoneButton.setEnabled(enableSelectNone);
+		selectNoneButton.setEnabled(pingPongList.getRightSize() != 0);
 		buttonsBox.add(selectNoneButton);
 		buttonsBox.add(createHorizontalStrut(12));
 		
@@ -159,24 +154,24 @@ public class EncodingsOptionPane extends AbstractOptionPane
 	@Override
 	protected void _save()
 	{
-		
 		jEdit.setProperty("buffer."+ JEditBuffer.ENCODING,(String)
 			defaultEncoding.getSelectedItem());
 		jEdit.setBooleanProperty("buffer.encodingAutodetect",
 			encodingAutodetect.isSelected());
 		jEdit.setProperty("encodingDetectors",encodingDetectors.getText());
 		jEdit.setProperty("fallbackEncodings",fallbackEncodings.getText());
-		
-		for (Entry entry : encodingsList.getValues())
+		Iterator<String> available = pingPongList.getLeftDataIterator();
+		while (available.hasNext())
 		{
-			if (entry.isChecked())
-			{
-				unsetProperty("encoding.opt-out."+entry.getValue());
-			}
-			else
-			{
-				setBooleanProperty("encoding.opt-out."+entry.getValue(),true);
-			}
+			String encoding = available.next();
+			setBooleanProperty("encoding.opt-out."+encoding,true);
+
+		}
+		Iterator<String> selected = pingPongList.getRightDataIterator();
+		while (selected.hasNext())
+		{
+			String encoding = selected.next();
+			unsetProperty("encoding.opt-out."+encoding);
 		}
 	} //}}}
 
@@ -190,43 +185,13 @@ public class EncodingsOptionPane extends AbstractOptionPane
 			Object source = ae.getSource();
 			if (source == selectAllButton)
 			{
-				encodingsList.selectAll();
+				pingPongList.moveAllToRight();
 			}
 			else if (source == selectNoneButton)
 			{
-				for (int i=0, c=encodingsList.getRowCount() ; i<c ; i++)
-				{
-					encodingsList.setValueAt(false,i,0);
-				}
-			}
+				pingPongList.moveAllToLeft();
+			}      
 		}
 	} //}}}
-
-	//{{{ TableModelHandler class
-	private class TableModelHandler implements TableModelListener
-	{
-		public void tableChanged(TableModelEvent tme)
-		{
-			int checkedAmount = encodingsList.getCheckedValues().length;
-			if (checkedAmount == 0)
-			{
-				selectNoneButton.setEnabled(false);
-			}
-			else
-			{
-				selectNoneButton.setEnabled(true);
-			}
-			if (encodingsList.getValues().length == checkedAmount)
-			{
-				selectAllButton.setEnabled(false);
-			}
-			else
-			{
-				selectAllButton.setEnabled(true);
-			}
-		}
-	} //}}}
-
 	//}}}
-
 } //}}}
