@@ -30,6 +30,7 @@ import org.gjt.sp.jedit.io.VFS;
 import org.gjt.sp.jedit.io.VFSManager;
 import org.gjt.sp.util.Log;
 import org.gjt.sp.util.StandardUtilities;
+import org.gjt.sp.util.ThreadUtilities;
 import org.gjt.sp.util.XMLUtilities;
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
@@ -58,7 +59,7 @@ import java.util.List;
 //}}}
 
 /**
- * @version $Id: InstallPanel.java 14125 2008-12-01 10:06:24Z kpouer $
+ * @version $Id: InstallPanel.java 18335 2010-08-12 09:25:34Z kpouer $
  */
 class InstallPanel extends JPanel implements EBComponent
 {
@@ -178,6 +179,9 @@ class InstallPanel extends JPanel implements EBComponent
 	*/
 	boolean loadPluginSet(String path)
 	{
+		pluginSet.clear();
+		pluginModel.restoreSelection(new HashSet<String>(), new HashSet<String>());
+
 		VFS vfs = VFSManager.getVFSForPath(path);
 		Object session = vfs.createVFSSession(path, InstallPanel.this);
 		try
@@ -203,7 +207,7 @@ class InstallPanel extends JPanel implements EBComponent
 		pluginModel.clear();
 		infoBox.setText(jEdit.getProperty("plugin-manager.list-download"));
 
-		VFSManager.runInAWTThread(new Runnable()
+		ThreadUtilities.runInDispatchThread(new Runnable()
 		{
 			public void run()
 			{
@@ -249,16 +253,6 @@ class InstallPanel extends JPanel implements EBComponent
 
 	//{{{ Inner classes
 
-	//{{{ KeyboardCommand enum
-	public enum KeyboardCommand
-	{
-		NONE,
-		TAB_OUT_FORWARD,
-		TAB_OUT_BACK,
-		EDIT_PLUGIN,
-		CLOSE_PLUGIN_MANAGER
-	} //}}}
-
 	//{{{ PluginTableModel class
 	private class PluginTableModel extends AbstractTableModel
 	{
@@ -300,7 +294,7 @@ class InstallPanel extends JPanel implements EBComponent
 				case 2: return ' '+jEdit.getProperty("install-plugins.info.category");
 				case 3: return ' '+jEdit.getProperty("install-plugins.info.version");
 				case 4: return ' '+jEdit.getProperty("install-plugins.info.size");
-				case 5: return ' '+"Release date";
+				case 5: return ' '+jEdit.getProperty("install-plugins.info.releaseDate");
 				default: throw new Error("Column out of range");
 			}
 		} //}}}
@@ -392,7 +386,6 @@ class InstallPanel extends JPanel implements EBComponent
 				return;
 
 			String[] args = { entry.name };
-
 			int result = GUIUtilities.listConfirm(
 				window,"plugin-manager.dependency",
 				args,parents);
@@ -419,8 +412,9 @@ class InstallPanel extends JPanel implements EBComponent
 				return;
 
 			Entry entry = (Entry)obj;
+			boolean before = entry.install;
 			entry.install = Boolean.TRUE.equals(aValue);
-
+			if (before == entry.install) return;
 			if (!entry.install)
 				deselectParents(entry);
 
@@ -558,51 +552,53 @@ class InstallPanel extends JPanel implements EBComponent
 		{
 			for (int i=0, c=getRowCount() ; i<c ; i++)
 			{
-				String name = entries.get(i).toString();
+				Object obj = entries.get(i);
+				String name = obj.toString();
+				if (obj instanceof Entry) {
+					name = ((Entry)obj).plugin.jar;
+				}
 				if (pluginSet.contains(name))
 					setValueAt(true, i, 0);
 				else setValueAt(savedChecked.contains(name), i, 0);
 			}
-
-			if (null != table)
+			if (table == null) return;
+			
+			table.setColumnSelectionInterval(0,0);
+			if (!savedSelection.isEmpty())
 			{
-				table.setColumnSelectionInterval(0,0);
-				if (!savedSelection.isEmpty())
+				int i = 0;
+				int rowCount = getRowCount();
+				for ( ; i<rowCount ; i++)
 				{
-					int i = 0;
-					int rowCount = getRowCount();
-					for ( ; i<rowCount ; i++)
+					String name = entries.get(i).toString();
+					if (savedSelection.contains(name))
 					{
-						String name = entries.get(i).toString();
-						if (savedSelection.contains(name))
-						{
-							table.setRowSelectionInterval(i,i);
-							break;
-						}
-					}
-					ListSelectionModel lsm = table.getSelectionModel();
-					for ( ; i<rowCount ; i++)
-					{
-						String name = entries.get(i).toString();
-						if (savedSelection.contains(name))
-						{
-							lsm.addSelectionInterval(i,i);
-						}
+						table.setRowSelectionInterval(i,i);
+						break;
 					}
 				}
-				else
+				ListSelectionModel lsm = table.getSelectionModel();
+				for ( ; i<rowCount ; i++)
 				{
-					if (table.getRowCount() != 0)
-						table.setRowSelectionInterval(0,0);
-					JScrollBar scrollbar = scrollpane.getVerticalScrollBar();
-					scrollbar.setValue(scrollbar.getMinimum());
+					String name = entries.get(i).toString();
+					if (savedSelection.contains(name))
+					{
+						lsm.addSelectionInterval(i,i);
+					}
 				}
 			}
+			else
+			{
+				if (table.getRowCount() != 0)
+					table.setRowSelectionInterval(0,0);
+				JScrollBar scrollbar = scrollpane.getVerticalScrollBar();
+				scrollbar.setValue(scrollbar.getMinimum());
+			}
 		} //}}}
 	} //}}}
 
 	//{{{ Entry class
-	private class Entry
+	private static class Entry
 	{
 		String name, installedVersion, version, author, date, description, set;
 
@@ -791,7 +787,7 @@ class InstallPanel extends JPanel implements EBComponent
 		{
 			if (localName.equals("plugin"))
 			{
-				pluginSet.add(attrs.getValue("name"));
+				pluginSet.add(attrs.getValue("jar"));
 			}
 		}
 	} //}}}
@@ -800,7 +796,7 @@ class InstallPanel extends JPanel implements EBComponent
 	private class ChoosePluginSet extends RolloverButton implements ActionListener
 	{
 		private String path;
-		
+
 		//{{{ ChoosePluginSet constructor
 		ChoosePluginSet()
 		{
@@ -822,11 +818,13 @@ class InstallPanel extends JPanel implements EBComponent
 		//{{{ actionPerformed() method
 		public void actionPerformed(ActionEvent ae)
 		{
+			
 			path = jEdit.getProperty(PluginManager.PROPERTY_PLUGINSET,
 				jEdit.getSettingsDirectory() + File.separator);
 			String[] selectedFiles = GUIUtilities.showVFSFileDialog(InstallPanel.this.window,
 				jEdit.getActiveView(), path, VFSBrowser.OPEN_DIALOG, false);
 			if (selectedFiles == null || selectedFiles.length != 1) return;
+
 			path = selectedFiles[0];
 			boolean success = loadPluginSet(path);
 			if (success)
@@ -847,7 +845,7 @@ class InstallPanel extends JPanel implements EBComponent
 			setToolTipText("clear plugin set");
 			addActionListener(this);
 		} //}}}
-		
+
 		//{{{ actionPerformed() method
 		public void actionPerformed(ActionEvent e)
 		{
@@ -1152,5 +1150,5 @@ class InstallPanel extends JPanel implements EBComponent
 	//}}}
 
 	static final Icon ASC_ICON  = GUIUtilities.loadIcon("arrow-asc.png");
-	static final Icon DESC_ICON = GUIUtilities.loadIcon("arrow-desc.png");	
+	static final Icon DESC_ICON = GUIUtilities.loadIcon("arrow-desc.png");
 }
