@@ -35,7 +35,7 @@ import org.gjt.sp.util.Log;
  * 
  * @since jEdit 4.2pre1
  * @author Slava Pestov
- * @version $Id: DisplayManager.java 16114 2009-08-29 17:53:21Z kpouer $
+ * @version $Id: DisplayManager.java 20040 2011-10-02 01:25:02Z ezust $
  */
 public class DisplayManager
 {
@@ -289,112 +289,18 @@ public class DisplayManager
 	 * Expands the fold at the specified physical line index.
 	 * @param line A physical line index
 	 * @param fully If true, all subfolds will also be expanded
+	 * @return the line number of the first subfold, or -1 if none
 	 * @since jEdit 4.2pre1
 	 */
 	public int expandFold(int line, boolean fully)
 	{
-		// the first sub-fold. used by JEditTextArea.expandFold().
-		int returnValue = -1;
-
-		int lineCount = buffer.getLineCount();
-		int end = lineCount - 1;
-
-		if (line == lineCount - 1)
-		{
-			return -1;
-		}
-		while (!isLineVisible(line))
-		{
-			int prevLine = folds.lookup(folds.search(line)) - 1;
-			if (!isLineVisible(prevLine))
-			{
-				return -1;
-			}
-			expandFold(prevLine, fully);
-			if (!isLineVisible(prevLine + 1))
-			{
-				return -1;
-			}
-		}
-		if (isLineVisible(line+1) && !fully)
-		{
-			return -1;
-		}
-
-		//{{{ Find fold start and fold end...
-		int start;
-		int initialFoldLevel = buffer.getFoldLevel(line);
-		if (buffer.getFoldLevel(line + 1) > initialFoldLevel)
-		{
-			// this line is the start of a fold
-			start = line;
-			if (!isLineVisible(line + 1) && folds.search(line + 1) != folds.count() - 1)
-			{
-				int index = folds.search(line + 1);
-				end = folds.lookup(index + 1) - 1;
-			}
-			else
-			{
-				for (int i = line + 1; i < lineCount; i++)
-				{
-					if (buffer.getFoldLevel(i) <= initialFoldLevel)
-					{
-						end = i - 1;
-						break;
-					}
-				}
-			}
-		}
-		else
-		{
-			if (!fully)
-			{
-				return -1;
-			}
-			start = line;
-			while (start > 0 && buffer.getFoldLevel(start) >= initialFoldLevel)
-			{
-				start--;
-			}
-			initialFoldLevel = buffer.getFoldLevel(start);
-			for (int i = line + 1; i < lineCount; i++)
-			{
-				if (buffer.getFoldLevel(i) <= initialFoldLevel)
-				{
-					end = i - 1;
-					break;
-				}
-			}
-		} // }}}
-
-		//{{{ Expand the fold...
-		if(fully)
-		{
-			showLineRange(start,end);
-		}
-		else
-		{
-			for (int i = start + 1; i <= end;)
-			{
-				if (returnValue == -1 && buffer.isFoldStart(i))
-				{
-					returnValue = i;
-				}
-
-				showLineRange(i, i);
-				int fold = buffer.getFoldLevel(i);
-				i++;
-				while (i <= end && buffer.getFoldLevel(i) > fold)
-				{
-					i++;
-				}
-			}
-		} // }}}
-
-		notifyScreenLineChanges();
-		textArea.foldStructureChanged();
-
-		return returnValue;
+		MutableInteger firstSubfold = new MutableInteger(-1);
+		boolean unfolded = _expandFold(line, fully, firstSubfold);
+		
+		if (unfolded)
+			textArea.foldStructureChanged();
+		
+		return firstSubfold.get();
 	} //}}}
 
 	//{{{ expandAllFolds() method
@@ -437,14 +343,21 @@ public class DisplayManager
 
 		showLineRange(0,buffer.getLineCount() - 1);
 
-		/* this ensures that the first line is always visible */
-		boolean seenVisibleLine = false;
-
+		int leastFolded = -1;
 		int firstInvisible = 0;
 
 		for(int i = 0; i < buffer.getLineCount(); i++)
 		{
-			if(!seenVisibleLine || buffer.getFoldLevel(i) < foldLevel)
+			// Keep track of the least fold level up to this point in the file,
+			// because we can't hide a line at this level since there will be no "root"
+			// line to unfold it from
+			if (leastFolded == -1 || buffer.getFoldLevel(i) < leastFolded)
+			{
+				leastFolded = buffer.getFoldLevel(i);
+			}
+		
+			if (buffer.getFoldLevel(i) < foldLevel ||
+			    buffer.getFoldLevel(i) == leastFolded)
 			{
 				if(firstInvisible != i)
 				{
@@ -452,7 +365,6 @@ public class DisplayManager
 						i - 1);
 				}
 				firstInvisible = i + 1;
-				seenVisibleLine = true;
 			}
 		}
 
@@ -864,5 +776,147 @@ public class DisplayManager
 		scrollLineCount.callChanged = true;
 	} //}}}
 
+	//{{{ _expandFold() method
+	/**
+	 * Expands the fold at the specified physical line index.
+	 * @param line A physical line index
+	 * @param fully If true, all subfolds will also be expanded
+	 * @param firstSubfold Will be set to the line number of the first
+	 * subfold, or -1 if there is none.
+	 * @return True if some line was unfolded, false otherwise.
+	 */
+	private boolean _expandFold(int line, boolean fully, MutableInteger firstSubfold)
+	{
+		boolean unfolded = false;
+
+		int lineCount = buffer.getLineCount();
+		int end = lineCount - 1;
+
+		if (line == lineCount - 1)
+		{
+			return false;
+		}
+		while (!isLineVisible(line))
+		{
+			int prevLine = folds.lookup(folds.search(line)) - 1;
+			if (!isLineVisible(prevLine))
+			{
+				return unfolded;
+			}
+			
+			// If any fold farther down was unfolded, then the text
+			// area needs to be updated
+			unfolded |= _expandFold(prevLine, fully, firstSubfold);
+			
+			if (!isLineVisible(prevLine + 1))
+			{
+				return unfolded;
+			}
+		}
+		if (isLineVisible(line+1) && !fully)
+		{
+			return unfolded;
+		}
+
+		//{{{ Find fold start and fold end...
+		int start;
+		int initialFoldLevel = buffer.getFoldLevel(line);
+		if (buffer.getFoldLevel(line + 1) > initialFoldLevel)
+		{
+			// this line is the start of a fold
+			start = line;
+			if (!isLineVisible(line + 1) && folds.search(line + 1) != folds.count() - 1)
+			{
+				int index = folds.search(line + 1);
+				end = folds.lookup(index + 1) - 1;
+			}
+			else
+			{
+				for (int i = line + 1; i < lineCount; i++)
+				{
+					if (buffer.getFoldLevel(i) <= initialFoldLevel)
+					{
+						end = i - 1;
+						break;
+					}
+				}
+			}
+		}
+		else
+		{
+			if (!fully)
+			{
+				return unfolded;
+			}
+			start = line;
+			while (start > 0 && buffer.getFoldLevel(start) >= initialFoldLevel)
+			{
+				start--;
+			}
+			initialFoldLevel = buffer.getFoldLevel(start);
+			for (int i = line + 1; i < lineCount; i++)
+			{
+				if (buffer.getFoldLevel(i) <= initialFoldLevel)
+				{
+					end = i - 1;
+					break;
+				}
+			}
+		} // }}}
+
+		//{{{ Expand the fold...
+		if(fully)
+		{
+			showLineRange(start,end);
+		}
+		else
+		{
+			boolean foundSubfold = false;
+			for (int i = start + 1; i <= end;)
+			{
+				if (!foundSubfold && buffer.isFoldStart(i))
+				{
+					firstSubfold.set(i);
+					foundSubfold = true;
+				}
+
+				showLineRange(i, i);
+				int fold = buffer.getFoldLevel(i);
+				i++;
+				while (i <= end && buffer.getFoldLevel(i) > fold)
+				{
+					i++;
+				}
+			}
+		}
+		
+		unfolded = true;
+		// }}}
+		
+		notifyScreenLineChanges();
+		return unfolded;
+	} //}}}
+	
+	//{{{ MutableInteger class
+	private class MutableInteger
+	{
+		MutableInteger(int value)
+		{
+			this.value = value;
+		}
+		
+		public void set(int value)
+		{
+			this.value = value;
+		}
+		
+		public int get()
+		{
+			return value;
+		}
+		
+		private int value;
+	} //}}}
+	
 	//}}}
 }
