@@ -43,7 +43,7 @@ import java.util.List;
 
 /**
  * @author Slava Pestov
- * @version $Id: TextAreaTransferHandler.java 16348 2009-10-14 10:40:15Z kpouer $
+ * @version $Id: TextAreaTransferHandler.java 19550 2011-06-08 21:37:11Z ezust $
  */
 public class TextAreaTransferHandler extends TransferHandler
 {
@@ -208,11 +208,11 @@ public class TextAreaTransferHandler extends TransferHandler
 			for (int i = 0;i<components.length;i++)
 			{
 				String str0 = components[i];
-				
+
 				if (str0.length() > 0)
 				{
 					URI uri = new URI(str0); // this handles the URI-decoding
-					
+
 					if ("file".equals(uri.getScheme()))
 					{
 						File file = new File(uri.getPath());
@@ -244,7 +244,7 @@ public class TextAreaTransferHandler extends TransferHandler
 					}
 				}
 			}
-			
+
 			if (found)
 			{
 				return true;
@@ -412,7 +412,19 @@ public class TextAreaTransferHandler extends TransferHandler
 		dragSource = null;
 	} //}}}
 
-	//{{{ canImport() method
+	//{{{ canImport() methods
+	@Override
+	public boolean canImport(TransferSupport support)
+	{
+		if (dragSource != null)
+			return true;
+		else
+		{
+			support.setDropAction(COPY);
+			return super.canImport(support);
+		}
+	}
+
 	@Override
 	public boolean canImport(JComponent c, DataFlavor[] flavors)
 	{
@@ -435,13 +447,6 @@ public class TextAreaTransferHandler extends TransferHandler
 				if(textArea.isEditable())
 					returnValue = true;
 			}
-			else if (flavors[i].getRepresentationClass().equals(java.io.InputStream.class))
-			{
-				//workaround for Ubuntu/Gnome/Nautilus import flavors, otherwise
-				//doesn't work on all Ubuntu installations
-				returnValue = true;
-				break;
-			}
 		}
 
 		Log.log(Log.DEBUG,this,"canImport() returning "
