@@ -27,6 +27,10 @@ package org.gjt.sp.util;
 
 //{{{ Imports
 import javax.swing.text.Segment;
+
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
+import java.text.DecimalFormat;
 import java.util.Comparator;
 import java.util.Stack;
 //}}}
@@ -35,15 +39,12 @@ import java.util.Stack;
  * Several tools that depends on JDK only.
  *
  * @author Matthieu Casanova
- * @version $Id: StandardUtilities.java 14923 2009-04-13 18:40:55Z shlomy $
+ * @version $Id: StandardUtilities.java 18200 2010-07-13 19:38:43Z daleanson $
  * @since 4.3pre5
  */
 public class StandardUtilities
 {
-
-	//{{{ Text methods
-
-	//{{{ charsToEscapes() method
+	//{{{ charsToEscapes() methods
 	/**
 	 * Escapes newlines, tabs, backslashes, and quotes in the specified
 	 * string.
@@ -53,9 +54,8 @@ public class StandardUtilities
 	public static String charsToEscapes(String str)
 	{
 		return charsToEscapes(str,"\n\t\\\"'");
-	} //}}}
+	}
 
-	//{{{ charsToEscapes() method
 	/**
 	 * Escapes the specified characters in the specified string.
 	 * @param str The string
@@ -106,7 +106,7 @@ public class StandardUtilities
 
 	} //}}}
 
-	//{{{ getLeadingWhiteSpace() method
+	//{{{ getLeadingWhiteSpace() methods
 	/**
 	 * Returns the number of leading white space characters in the
 	 * specified string.
@@ -116,9 +116,8 @@ public class StandardUtilities
 	public static int getLeadingWhiteSpace(String str)
 	{
 		return getLeadingWhiteSpace((CharSequence)str);
-	} //}}}
+	}
 
-	//{{{ getLeadingWhiteSpace() method
 	/**
 	 * Returns the number of leading white space characters in the
 	 * specified string.
@@ -168,7 +167,7 @@ loop:		for(int i = str.length() - 1; i >= 0; i--)
 		return whitespace;
 	} //}}}
 
-	//{{{ getLeadingWhiteSpaceWidth() method
+	//{{{ getLeadingWhiteSpaceWidth() methods
 	/**
 	 * Returns the width of the leading white space in the specified
 	 * string.
@@ -178,9 +177,8 @@ loop:		for(int i = str.length() - 1; i >= 0; i--)
 	public static int getLeadingWhiteSpaceWidth(String str, int tabSize)
 	{
 		return getLeadingWhiteSpaceWidth((CharSequence)str, tabSize);
-	} //}}}
+	}
 
-	//{{{ getLeadingWhiteSpaceWidth() method
 	/**
 	 * Returns the width of the leading white space in the specified
 	 * string.
@@ -705,6 +703,52 @@ loop:		for(int i = 0; i < str.length(); i++)
 		return def;
 	} //}}}
 
+	//{{{ formatFileSize() method
+	public static final DecimalFormat KB_FORMAT = new DecimalFormat("#.# kB");
+	public static final DecimalFormat MB_FORMAT = new DecimalFormat("#.# MB");
+
+	/**
+	 * Formats the given file size into a nice string (123 Bytes, 10.6 kB,
+	 * 1.2 MB).
+	 * @param length The size
+	 * @since jEdit 4.4pre1
+	 */
+	public static String formatFileSize(long length)
+	{
+		if(length < 1024)
+		{
+			return length + " Bytes";
+		}
+		else if(length < 1024 << 10)
+		{
+			return KB_FORMAT.format((double)length / 1024);
+		}
+		else
+		{
+			return MB_FORMAT.format((double)length / 1024 / 1024);
+		}
+	} //}}}
+
 	//}}}
 	private StandardUtilities(){}
+
+	// {{{ MD5 sum method
+	/**
+	 * Returns the md5sum for given string. Or dummy byte array on error
+	 * Supress NoSuchAlgorithmException because MD5 algorithm always present in JRE
+	 * @param s Given string
+	 * @return md5 sum of given string 
+	 */
+	public static byte[] md5(String s) {
+		final byte[] dummy = new byte[1];
+		try {
+			MessageDigest digest = MessageDigest.getInstance("MD5");
+			digest.update(s.getBytes());
+			return digest.digest();
+		} catch (NoSuchAlgorithmException e) {
+			Log.log(Log.ERROR, StandardUtilities.class, "Can't Calculate MD5 hash!", e);
+			return dummy;
+		}
+	}
+	// }}}
 }
